<?php
namespace nur\data\types;

use nur\b\ValueException;
use nur\t\TestCase;

class SDatetimeTypeTest extends TestCase {
  function test() {
    $type = new SDatetimeType();
    $year = date("Y");

    self::assertSame(null, $type->with(null));
    self::assertSame(false, $type->with(false));
    self::assertSame(null, $type->with(""));

    self::assertException(ValueException::class, [$type, "with"], "15 8");
    self::assertSame("15/01/$year 08:25:00", $type->with("15/1 8:25"));
    self::assertSame("15/01/1999 08:25:32", $type->with("15/1/99 8.25.32"));
    self::assertSame("15/01/2020 08:25:32", $type->with("15/1/20 08:25:32"));
    self::assertSame("16/01/2069 01:20:00", $type->with("15/1/69 25:20"));
    self::assertSame("15/01/1970 03:04:00", $type->with("15/1/70 3:4"));
    self::assertSame("15/01/1971 20:16:00", $type->with("15/1/71 20:16"));

    self::assertSame("15/12/2000 11:35:00", $type->with("15/0/2001 11:35"));
    self::assertSame("15/01/2002 11:35:00", $type->with("15/13/2001 11:35"));
    self::assertSame("31/05/2001 11:35:00", $type->with("0/6/2001 11:35"));
    self::assertSame("01/07/2001 11:35:00", $type->with("31/6/2001 11:35"));

    self::assertSame("12/03/2020 12:42:00", $type->with(" 12/03/2020   12:42   "));
    self::assertException(ValueException::class, [$type, "with"], " 12/03/2020 1");
    self::assertSame("12/03/2020 01:02:00", $type->with(" 12/03/2020 1:2"));
    self::assertSame("12/03/2020 01:02:03", $type->with(" 12/03/2020 1:2:3"));

    self::assertException(ValueException::class, [$type, "with"], "15");
    self::assertSame("15/01/$year 00:00:00", $type->with("15/1"));
    self::assertSame("15/01/1999 00:00:00", $type->with("15/1/99"));
    self::assertSame("15/01/2020 00:00:00", $type->with("15/1/20"));
    self::assertSame("15/01/2069 00:00:00", $type->with("15/1/69"));
    self::assertSame("15/01/1970 00:00:00", $type->with("15/1/70"));
    self::assertSame("15/01/1971 00:00:00", $type->with("15/1/71"));

    self::assertSame("15/12/2000 00:00:00", $type->with("15/0/2001"));
    self::assertSame("15/01/2002 00:00:00", $type->with("15/13/2001"));
    self::assertSame("31/05/2001 00:00:00", $type->with("0/6/2001"));
    self::assertSame("01/07/2001 00:00:00", $type->with("31/6/2001"));

    self::assertSame("12/03/2020 00:00:00", $type->with(" 12/03/2020 "));
    self::assertException(ValueException::class, [$type, "with"], " 12/03/2020 0");
    self::assertSame("12/03/2020 00:00:00", $type->with(" 12/03/2020 0:0"));
    self::assertSame("12/03/2020 00:00:00", $type->with(" 12/03/2020 0:0:0"));
  }
}
